# frozen_string_literal: true

require "core/extension"

require_relative "../core/local/store"

module Is
  # [public] Manage thread-local program state for an object.
  #
  module Localized
    extend Is::Extension

    # [public] Localize `value` for `key` in the current thread or fiber.
    #
    # Be aware that you can easily introduce memory leaks into your program if you aren't careful to clean up localized
    # state. There's two ways to approach this:
    #
    #   1. Pass a block to `localize`. The value will be localized and cleaned up after being yielded to the block.
    #
    #   2. Call `unlocalize` manually when you no longer need the localized value.
    #
    def localize(key, value)
      key = localized_key(key)

      Core::Local::Store[key] = value

      if block_given?
        begin
          yield value
        ensure
          Core::Local::Store.delete(key)
        end
      else
        value
      end
    end

    # [public] Returns the localized value for `key`, or `fallback`.
    #
    def localized(key, fallback = nil)
      Core::Local::Store.fetch(localized_key(key), fallback)
    end

    # [public] Deletes the localized value for `key`.
    #
    def unlocalize(key)
      Core::Local::Store.delete(localized_key(key))
    end

    private def localized_key(name)
      "#{localized_key_prefix}_#{name}"
    end

    private def localized_key_prefix
      @__localized_key_prefix ||= "__corerb_localized_#{object_id}"
    end
  end
end
